/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* user space process that tracks /dev/iptrk
*
**************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <fcntl.h>
#include <pwd.h>
#include <time.h>
#include <dirent.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <endian.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <linux/if_ether.h>
#include "../version.h"

#define PRIVATE static
#define PUBLIC

#define IPTRK_PROC_PATH       "/proc/iptrk"

#define IPTRK_CONF            IPTRK_PATH     "/conf"
#define IPTRK_LS_CONF         IPTRK_PATH     "/listserv"

#define IPTRK_DB_WATCH_PATH   IPTRK_PATH     "/db_watch"
#define IPTRK_DB_WATCH_PATHC  IPTRK_PATH     "/db_watch/current"

#define IPTRK_LOG             IPTRK_PATH     "/log"
#define IPTRK_TMP             IPTRK_PATH     "/tmp"

#define IPTRK_LOG_IP_PATH     IPTRK_PATH     "/log_ip"
#define IPTRK_LOG_IP_PATHC    IPTRK_PATH     "/log_ip/current"

#define IPTRK_LOG_PORT_PATH   IPTRK_PATH     "/log_port"
#define IPTRK_LOG_PORT_FILEC  IPTRK_PATH     "/log_port/current"

#define IPTRK_GRAPHS_ABS                     "/graphs"
#define IPTRK_GRAPHS_PATH     IPTRK_PATH     IPTRK_GRAPHS_ABS
#define TTF_FONT              IPTRK_PATH     "/font.ttf"

#define BACKUP_SCRIPT         IPTRK_PATH     "/backup.sh"
#define ARCHIVE_SCRIPT        IPTRK_PATH     "/archive.sh"

#define U_8            unsigned char
#define U_16           unsigned short int
#define S_16           short int
#define U_32           unsigned long int
#define S_32           long int
#define U_64           unsigned long long
#define S_64           long long

#define STR_LEN        256
#define STR_SMALL      40
#define STR_LEN_HUGE   65536

#define COLOUR_NONE     0
#define COLOUR_TOPBAR   1
#define COLOUR_TOPBAR2  2
#define COLOUR_TOPBARO  3
#define COLOUR_EDIT     3
#define COLOUR_TOPBAR2O 4
#define COLOUR_NORMAL   5
#define COLOUR_HELP     6
#define COLOUR_ALERT    7
#define COLOUR_WHITE    8
#define COLOUR_RED      9
#define COLOUR_BLUE     10
#define COLOUR_YELLOW   11
#define COLOUR_CYAN     12
#define COLOUR_MAGENTA  13
#define COLOUR_GREEN    14
#define SF_MAX          8

#ifndef CTRL
#define CTRL(x)         ((x) & 0x1f)
#endif

#define KEY_TAB         9
#define KEY_CR          13
#define KEY_BSPACE      127

#define IPTRK_IF_SIZE 8

/********************************************************
* user side queue struct
********************************************************/
typedef struct
{
  char  dev[IPTRK_IF_SIZE + 1];
  U_32  src;
  U_32  dst;
  U_16  vol;
  U_8   prot;
  U_16  port_s;  /*only applic. to tcp and udp protocols*/
  U_16  port_d;  /*...*/
} iptrk_queue_t;

/********************************************************
* atoms
********************************************************/
typedef struct _ll_t
{
  struct _ll_t    *next;
} ll_t;

typedef struct _treap_t
{
  int             p;
  struct _treap_t *l;
  struct _treap_t *r;
} treap_t;

#define LL_ENTRY(n)    ((void *)n + sizeof(ll_t))
#define TREAP_ENTRY(n) ((void *)n + sizeof(treap_t))

/********************************************************
*
********************************************************/
#define BOOT_T_NIFS       8
#define BOOT_T_EMAIL_LIM  4
#define BOOT_T_CANCEL_LIM 8

#define DIR_m             0
#define DIR_I             1
#define DIR_O             2
#define DIR_u             3
#define DIR_i             4
#define DIR_o             5

#define INTERVAL_H        0
#define INTERVAL_D        1
#define INTERVAL_M        2

#define COND_LESS         0
#define COND_MORE         1

#define PROT_TCP          0
#define PROT_UDP          1
#define PROT_OTH          2
#define PROT_ALL          3

typedef struct
{
  U_32 addr;
  U_32 mask;
} addr_t;

typedef struct
{
  char   dev[STR_SMALL + 1];
  addr_t addr;
} our_net_t;

typedef struct
{
  unsigned dir  : 4;
  unsigned cond : 1;
  unsigned prot : 2;

  int      no;
  addr_t   addr;
  int      service;
  U_64     limit;
  int      zone;
  char     str[STR_LEN + 1];
} watch_t;

typedef struct
{
  unsigned prot : 2;

  int      no;
  addr_t   addr;
  int      service;
  int      zone;
  char     str[STR_LEN + 1];
} watch_private_t;

typedef struct
{
  U_16 port;
} eserv_t;

typedef struct
{
  int no;
  int day_s;
  int day_e;
  int hr_s;
  int hr_e;
} zone_t;

typedef struct
{
  int  no;
  int  limit;
  int  zone;
  char str[STR_LEN + 1];
} dosa_t;

typedef struct
{
  unsigned purge        : 2;
  unsigned archive      : 2;
  unsigned cancel_gram  : 1;

  int      wait_delay;
  char     nifs[BOOT_T_NIFS][STR_LEN + 1];
  int      watch_time; /*per hour*/
  char     email[BOOT_T_EMAIL_LIM][STR_LEN + 1];
  long     db_limit;
  ll_t     *our_net_ll;
  treap_t  *eserv_bt;
  treap_t  *zone_bt;
  ll_t     *dosa_addr_ll;
  ll_t     *dosa_port_ll;
  ll_t     *watch_ll;
  ll_t     *watch_default_ll;
  ll_t     *watch_private_ll;
  ll_t     *extra_report_ll;
  treap_t  *debug_bt;
} boot_t;

/********************************************************
*
********************************************************/
typedef union
{
  struct
  {
#if(__BYTE_ORDER == __LITTLE_ENDIAN)
    U_16 d;
    U_16 s;
#endif
  }      port;
  U_32   key;
} port_key_t;

typedef union
{
  struct
  {
#if(__BYTE_ORDER == __LITTLE_ENDIAN)
    U_32 d;
    U_32 s;
#endif
  }      addr;
  U_64   key;
} addr_key_t;

typedef struct
{
  port_key_t port;
  U_64       vol;
} port_t;

typedef struct
{
  addr_key_t addr;
  treap_t    *oth_port_ll;
  treap_t    *tcp_port_ll;
  treap_t    *udp_port_ll;
  U_64       vol;
  time_t     created;      /*time stamps*/
  time_t     last_updated;
} iptrk_t;

/********************************************************
*
********************************************************/
typedef struct /*protocol structure used in rt_track* code*/
{
  U_64 tot;
  U_64 oth;
  U_64 tcp;
  U_64 udp;
} volp_t;

typedef struct /*for general volume casting (used by rt_tracking.c)*/
{
  U_64 I;  /*external in*/
  U_64 O;  /*external out*/
  U_64 u;  /*internal (all types of internal)*/
  U_64 m;  /*martian packet volume*/
} voldg_t;

typedef struct /*for report type volume casting (maint.c)*/
{
  U_64 I; /*external in*/
  U_64 O; /*external out*/
  U_64 u; /*internal including in, out and packets looped back onthemselves*/
  U_64 i; /*internal in*/
  U_64 o; /*internal out*/
} voldr_t;

/********************************************************
* cgi stuff
********************************************************/
typedef struct
{   
  time_t time_stamp;
  char   name[STR_LEN + 1];
} dir_entry_t;
